
author: Tero Piirainen
date: 2015-01-21
title: From frameworkless JavaScript to Riot 2.0 and beyond
description: The story behind Riot 2.0 and issues we faced with Facebook React
parallax: /blog/img/slack/muut-slack-hero-bg.png
hero: /blog/img/slack/muut-slack-hero.png
muut_path: :riotjs20
hidden: true

====

## 1. Frameworkless JavaScript

Our highly popular [blog entry](/blog/technology/frameworkless-javascript.html) was the starting point for our minimalistic explorations. The point of that writing is that Muut cannot be build on top of the popular frameworks; Backbone, Angular or Ember.

![](/blog/img/no-framework/scrapped.jpg | 500 .bottom-up )

Main reasons:

1. We want our client application to be small. It's completely unacceptable that the underlying framework is bigger than the application itself.

2. We want a small framework API that we understand completely. We want to make sense from the stack traces. All the popular frameworks have too many API methods, properties and proprietary idioms. This is especially important for our special, fully embeddable client.

To some extend we were questioning whether anyone would need a framework at all. As long as the API (or model) is separated from UI the biggest troubles are gone. With events you can build isolated components and native `pushState` gives us the ability to deal with back button.

This reasoning worked as a basis for Riot.


## 2. Riot 1.0

Riot was proof of concept for the blog entry. What would be the most minimalistic MVP (model-view-presenter) stack? It started out as a side project but the educational trip became more serious towards the end. Eventually Riot 1.0 was out with following features:

1. Model View Presenter (MPV) pattern for structuring applications.

2. An `observable` interface for separating API (model) from the views and building isolated modules with events.

3. A router that takes care of URL and back button.

![](/riotjs/img/mvp.png)

Riot 1.0 was a big success. Obviously not for everyone since it was so much different from others. It was either loved or hated.

[Riot 1.0 blog entry](/blog/technology/riotjs-the-1kb-mvp-framework.html).


## 3. React

React was different. The idea of having related layout and logic under the same component seemed like the right thing to do. An UI component is always a combination HTML and JavaScript so it makes perfect sense to tie these technologies together.

No need to remember which elements are bound to which JS event for example. All the dependencies are neatly in one place, the component.

> "Templates separate technologies, not concerns."

Thank you this insight, Facebook!

Components result to less application code, less DOM manipulation and jQuery selectors. The UI code becomes easier to understand. Ideally you can simply work with JavaScript and the view takes care of itself.


### Problems in React

Minified React weights 124K, which is too big for us. We cannot force our end users to load this library on the embedding sites. And big codebase is usually a sign of unneeded complexity.Batching, sub-tree rendering, dirty checking... There must be a simpler way.

The syntax is too verbose: too many React- specific idioms, too many colons and curly brackets, a big list of method names to remember. Just too much "boilerplate".

JSX is almost good, but we still wanted clearer separation for these technologies. We wanted a way o unify HTML and JS with less pain.

React has a proprietary event system and a list of "supported" HTML tags. We didn't like the extra magic and wanted to work with DOM just like before.


## 4. Riot 2.0

Maybe we can build something like React but without the above concerns? We knew wanted one ourselves. And so Riot 2.0 was born.

We borrowed ideas from Riot 1.0 (minimalism), React (virtual dom, components) and HTML5 components (custom tags).

We wanted a tiny API. Just HTML and JavaScript without `riot-` data attributes or other personal flavors.

We exceeded our goals. Riot 2.0 is much smaller and simpler than anticipated. Not just 10x smaller, but 24x smaller than React and with only 9 public methods on a complete application stack.


## What's next?

Upcoming versions of Riot will include following:

1) Browser compilation similar to what LESS or JSX do avoiding the build phase on the server. Our compiler is small and fast so it's not completely crazy to this on production as well.

2) Server side HTML generation. This hot topic nowadays and lets you build so called "isomorphic" applications. This should be fairly easy to implement for Riot 2.0 since HTML generation is already on the core.

3) Server side tests. Bigger test suite working on server and client (using jsdom or similar).

4) Tag option validation using HTML attributes. Something like this:

``` html
<my-tag option1="Object" option2="String:required">

</my-tag>
```

We are also planning to make performance comparisons to other frameworks.

Admittedly, the shift from 1.0 to 2.0 is big. But we believe that 2.0 is much better. Custom tags seem to work and we use them heavily on this production website.

You should definitely give them a shot.


### [Riot 2.0 website...](/riotjs/)




